// RF24Network - Version: 1.0.8
#include <RF24Network.h>
#include <RF24Network_config.h>
#include <Sync.h>

// RF24 - Version: 1.3.1
#include <RF24.h>
#include "RF24_config2.h"
#include <nRF24L01.h>
#include <printf.h>

#include <SPI.h>

RF24 radio(7,8);                    // nRF24L01(+) radio attached using Getting Started board 
RF24Network network(radio);          // Network uses that radio

const uint16_t this_node = 02;        // Address of our node in Octal format (Node01,Node02, ...)
const uint16_t other_node = 00;       // Address of the master node (Node00) in Octal format

const unsigned long interval = 3000; //ms  // How often to send 'hello world to the other unit
unsigned long interval_on;

unsigned long last_sent;             // When did we last send?
unsigned long packets_sent;          // How many have we sent already
unsigned long last_on;
unsigned long last_check;

struct nodos_a_central {                  // Structure of our payload
  unsigned int nodo;
  unsigned int movimiento;
  unsigned int humo;
  unsigned int humedad;
};

struct central_a_nodos {
  unsigned int mensaje;
};

#define pirPin 5   // INPUT PIR HC-S501
#define ledPin 4   // LED detección movimiento
#define relPin 3   // Pin para el rele
#define alaPin 2   // LED alarma activa
bool pirValue;     // Place to store read PIR Value
bool alarm_connected;
bool always_on;
bool always_off;
bool send_message;
bool lights_off;
bool send_alarm = true;
int counter_attempts_send;
const unsigned long max_attempts_send = 3;

#define noserial

void setup(void)
{
  #ifdef serial
  Serial.begin(9600);
  Serial.println("Test");
  #endif
 
  SPI.begin();
  while (radio.begin() == false) {
    delay(5000);  
  }

  radio.setPALevel(RF24_PA_MIN);
  radio.setDataRate(RF24_250KBPS);
  network.begin(/*channel*/ 90, /*node address*/ this_node);
  pinMode(pirPin, INPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(alaPin, OUTPUT);
  pinMode(relPin, OUTPUT);
  
  alarm_connected = true;
  always_on = false;
  always_off = false;
  send_message = false;
  lights_off = true;
  counter_attempts_send = 0;
  last_on = millis();
  last_check = millis();
  
  digitalWrite(relPin, LOW);
  digitalWrite(alaPin, HIGH); //Prueba
   
  interval_on = 30000;

}

void loop() {

  if(radio.failureDetected){ 
    radio.begin();                       // Attempt to re-configure the radio with defaults
    radio.setPALevel(RF24_PA_MIN);
    radio.setDataRate(RF24_250KBPS);
    radio.failureDetected = 0;           // Reset the detection value
    network.begin(/*channel*/ 90, /*node address*/ this_node);
    delay(1000);
    return;
  }

  unsigned long now = millis();
  if (now-last_check >= interval) {
    last_check = now;
    network.update();                               // Check the network regularly    
    while ( network.available() ) {                 // Is there any incoming data
      RF24NetworkHeader header_from_00;
      central_a_nodos central_a_nodos_t;
      network.read(header_from_00, &central_a_nodos_t, sizeof(central_a_nodos_t)); // Read the incoming data
      #ifdef serial
      Serial.print("Recibido:");
      Serial.println(central_a_nodos_t.mensaje);
      #endif
      switch (central_a_nodos_t.mensaje) {
        case 1:
          alarm_connected = true;
          digitalWrite(alaPin, HIGH);
          break;
        case 2:
          alarm_connected = false;
          digitalWrite(alaPin, LOW);
          break;
        case 3:
          always_on = true;
          digitalWrite(relPin, HIGH);
          lights_off = false;
          break;
        case 4:
          always_on = false;
          always_off = false;
          break;
        case 5:
          always_off = true;
          digitalWrite(relPin, LOW);
          lights_off = true;          
          break;
        default:
          // intervalo encendido en segundos
          if ((central_a_nodos_t.mensaje > 200) and (central_a_nodos_t.mensaje <= 260)) interval_on = (central_a_nodos_t.mensaje - 200) * 1000;
          // intervalo encendido en minutos
          if ((central_a_nodos_t.mensaje > 100) and (central_a_nodos_t.mensaje <= 110)) interval_on = (central_a_nodos_t.mensaje - 100) * 60 * 1000;
          break;
      }
      return;
    }
    if (send_message) {
      if (counter_attempts_send > max_attempts_send) {
        send_message = false;
        counter_attempts_send = 0;
      } else {
        nodos_a_central nodos_a_central_t = { this_node, counter_attempts_send, 0, 0};
        RF24NetworkHeader header(other_node);
        bool message_sent = network.write(header,&nodos_a_central_t,sizeof(nodos_a_central_t));
        if (message_sent) {
          #ifdef serial
          Serial.print("Enviado mensaje de alarma a la ");
          Serial.println(contador_intentos_envio);
          #endif
          send_message = false;
          counter_attempts_send = 0;
        } else {
          counter_attempts_send = counter_attempts_send + 1;
          #ifdef serial
          Serial.print("a a la ");
          Serial.println(counter_attemps_send);
          #endif
        }
        return;
      }
    }
  }
  
  pirValue = digitalRead(pirPin);
  digitalWrite(ledPin, pirValue);
  
  if (pirValue == HIGH) {
    last_on = now;
    if ((alarm_connected) and (send_alarm)) send_message = true;
    send_alarm = false;
    if (lights_off) {
      digitalWrite(relPin, HIGH);
      lights_off = false;
    }
  } 
  else {
    if (now-last_on >= interval_on) {
      send_alarm = true;
      if (not always_on) {
        digitalWrite(relPin, LOW);
        lights_off = true;
      }
    }
  }
}
